window.Gleam.initSearch([{"doc":"glimit","title":"glimit","content":"# glimit\n\n[![Package Version](https://img.shields.io/hexpm/v/glimit)](https://hex.pm/packages/glimit)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glimit/)\n[![test](https://github.com/nootr/glimit/actions/workflows/test.yml/badge.svg)](https://github.com/nootr/glimit/actions/workflows/test.yml)\n\nA framework-agnostic rate limiter for Gleam. 💫\n\n&gt; ⚠️  This library is still in development, use at your own risk.\n\n\n## Usage\n\nA very minimalistic example of how to use `glimit` would be the following snippet:\n\n```gleam\nimport glimit\n\nlet limiter =\n  glimit.new()\n  |&gt; glimit.per_second(2)\n  |&gt; glimit.identifier(fn(x) { x })\n  |&gt; glimit.handler(fn(_) { &quot;Stop!&quot; })\n  |&gt; glimit.build\n\nlet func =\n  fn(_) { &quot;OK&quot; }\n  |&gt; glimit.apply(limiter)\n\nfunc(&quot;🚀&quot;) // &quot;OK&quot;\nfunc(&quot;💫&quot;) // &quot;OK&quot;\nfunc(&quot;💫&quot;) // &quot;OK&quot;\nfunc(&quot;💫&quot;) // &quot;Stop!&quot;\nfunc(&quot;🚀&quot;) // &quot;OK&quot;\nfunc(&quot;🚀&quot;) // &quot;Stop!&quot;\n```\n\nA more practical example would be to use `glimit` to rate limit requests to a mist HTTP server:\n\n```gleam\nimport glimit\n\n\nfn handle_request(req: Request(Connection)) -&gt; Response(ResponseData) {\n  let index =\n    response.new(200)\n    |&gt; response.set_body(mist.Bytes(bytes_builder.new()))\n  let not_found =\n    response.new(404)\n    |&gt; response.set_body(mist.Bytes(bytes_builder.new()))\n\n  case request.path_segments(req) {\n    [] -&gt; index\n    _ -&gt; not_found\n  }\n}\n\nfn get_identifier(req: Request(Connection)) -&gt; Result(String, String) {\n  req.body\n  |&gt; get_client_info\n  |&gt; result.map(fn(client_info: ConnectionInfo) {\n    client_info.ip_address |&gt; string.inspect\n  })\n  |&gt; result.unwrap(&quot;unknown IP address&quot;)\n}\n\npub fn main() {\n  let rate_limit_reached = fn(_req) -&gt; {\n    response.new(429)\n    |&gt; response.set_body(mist.Bytes(bytes_builder.new()))\n  }\n\n  let limiter =\n    glimit.new()\n    |&gt; glimit.per_second(10)\n    |&gt; glimit.per_minute(100)\n    |&gt; glimit.per_hour(1000)\n    |&gt; glimit.identifier(get_identifier)\n    |&gt; glimit.handler(rate_limit_reached)\n    |&gt; glimit.build\n\n  let assert Ok(_) =\n    handle_request\n    |&gt; glimit.apply(limiter)\n    |&gt; mist.new\n    |&gt; mist.port(8080)\n    |&gt; mist.start_http\n\n  process.sleep_forever()\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/glimit&gt;.\n","url":"index.html"},{"doc":"glimit/utils","title":"now","content":"pub fn now() -&gt; Int\n Get the current time in epoch seconds.\nSynonyms:\nglimit/utils.now\nglimit/utils now","url":"glimit/utils.html#now"},{"doc":"glimit/utils","title":"glimit/utils","content":" A module containing utility functions.\n","url":"glimit/utils.html"},{"doc":"glimit/actor","title":"Message","content":"pub type Message(id) {\n  Shutdown\n  Hit(identifier: id, reply_with: Subject(Result(Nil, Nil)))\n}\n The messages that the actor can receive.\n\nShutdown\n Stop the actor.\n\nHit(identifier: id, reply_with: Subject(Result(Nil, Nil)))\n Mark a hit for a given identifier.\n\nSynonyms:\nglimit/actor.Message\nglimit/actor Message","url":"glimit/actor.html#Message"},{"doc":"glimit/actor","title":"hit","content":"pub fn hit(\n  subject: Subject(Message(a)),\n  identifier: a,\n) -&gt; Result(Nil, Nil)\n Log a hit for a given identifier.\n\nSynonyms:\nglimit/actor.hit\nglimit/actor hit","url":"glimit/actor.html#hit"},{"doc":"glimit/actor","title":"new","content":"pub fn new(\n  per_second: Option(Int),\n  per_minute: Option(Int),\n  per_hour: Option(Int),\n) -&gt; Result(Subject(Message(a)), Nil)\n Create a new rate limiter actor.\n\nSynonyms:\nglimit/actor.new\nglimit/actor new","url":"glimit/actor.html#new"},{"doc":"glimit/actor","title":"stop","content":"pub fn stop(subject: Subject(Message(a))) -&gt; Nil\n Stop the actor.\n\nSynonyms:\nglimit/actor.stop\nglimit/actor stop","url":"glimit/actor.html#stop"},{"doc":"glimit/actor","title":"glimit/actor","content":" The rate limiter actor.\n","url":"glimit/actor.html"},{"doc":"glimit","title":"RateLimiter","content":"pub type RateLimiter(a, b, id) {\n  RateLimiter(\n    subject: Subject(actor.Message(id)),\n    handler: fn(a) -&gt; b,\n    identifier: fn(a) -&gt; id,\n  )\n}\n A rate limiter.\n\nRateLimiter(\n  subject: Subject(actor.Message(id)),\n  handler: fn(a) -&gt; b,\n  identifier: fn(a) -&gt; id,\n)\n\n\nSynonyms:\nglimit.RateLimiter\nglimit RateLimiter","url":"glimit.html#RateLimiter"},{"doc":"glimit","title":"RateLimiterBuilder","content":"pub type RateLimiterBuilder(a, b, id) {\n  RateLimiterBuilder(\n    per_second: Option(Int),\n    per_minute: Option(Int),\n    per_hour: Option(Int),\n    identifier: Option(fn(a) -&gt; id),\n    handler: Option(fn(a) -&gt; b),\n  )\n}\n A builder for configuring the rate limiter.\n\nRateLimiterBuilder(\n  per_second: Option(Int),\n  per_minute: Option(Int),\n  per_hour: Option(Int),\n  identifier: Option(fn(a) -&gt; id),\n  handler: Option(fn(a) -&gt; b),\n)\n\n\nSynonyms:\nglimit.RateLimiterBuilder\nglimit RateLimiterBuilder","url":"glimit.html#RateLimiterBuilder"},{"doc":"glimit","title":"apply","content":"pub fn apply(\n  func: fn(a) -&gt; b,\n  limiter: RateLimiter(a, b, c),\n) -&gt; fn(a) -&gt; b\n Apply the rate limiter to a request handler or function.\n\nSynonyms:\nglimit.apply\nglimit apply","url":"glimit.html#apply"},{"doc":"glimit","title":"apply2","content":"pub fn apply2(\n  func: fn(a, b) -&gt; c,\n  limiter: RateLimiter(#(a, b), c, d),\n) -&gt; fn(a, b) -&gt; c\n Apply the rate limiter to a request handler or function with two arguments.\n\n Note: this function folds the two arguments into a tuple before passing them to the\n identifier or handler functions.\n\n # Example\n\n \n import glimit\n\n let limiter =\n   glimit.new()\n   |&gt; glimit.per_hour(1000)\n   |&gt; glimit.identifier(fn(i: #(String, String)) {\n     let #(a, _) = i\n     a\n   })\n   |&gt; glimit.handler(fn(_) { &quot;Rate limit reached&quot; })\n   |&gt; glimit.build()\n\n let handler =\n   fn(a, b) { a &lt;&gt; b }\n   |&gt; glimit.apply2(limiter)\n \nSynonyms:\nglimit.apply2\nglimit apply2","url":"glimit.html#apply2"},{"doc":"glimit","title":"apply3","content":"pub fn apply3(\n  func: fn(a, b, c) -&gt; d,\n  limiter: RateLimiter(#(a, b, c), d, e),\n) -&gt; fn(a, b, c) -&gt; d\n Apply the rate limiter to a request handler or function with three arguments.\n\n Note: this function folds the three arguments into a tuple before passing them to the\n identifier or handler functions.\n\nSynonyms:\nglimit.apply3\nglimit apply3","url":"glimit.html#apply3"},{"doc":"glimit","title":"apply4","content":"pub fn apply4(\n  func: fn(a, b, c, d) -&gt; e,\n  limiter: RateLimiter(#(a, b, c, d), e, f),\n) -&gt; fn(a, b, c, d) -&gt; e\n Apply the rate limiter to a request handler or function with four arguments.\n\n Note: this function folds the four arguments into a tuple before passing them to the\n identifier or handler functions.\n\n &gt; ⚠️ For functions with more than four arguments, you&#39;ll need to write a custom\n &gt; wrapper function that folds the arguments into a tuple before passing them to the\n &gt; rate limiter. This is because Gleam does not support variadic functions, because\n &gt; the BEAM VM identifies functions by their arity.\n\nSynonyms:\nglimit.apply4\nglimit apply4","url":"glimit.html#apply4"},{"doc":"glimit","title":"build","content":"pub fn build(\n  config: RateLimiterBuilder(a, b, c),\n) -&gt; RateLimiter(a, b, c)\n Build the rate limiter.\n\n Panics if the rate limiter actor cannot be started or if the identifier\n function or handler function is missing.\n\nSynonyms:\nglimit.build\nglimit build","url":"glimit.html#build"},{"doc":"glimit","title":"handler","content":"pub fn handler(\n  limiter: RateLimiterBuilder(a, b, c),\n  handler: fn(a) -&gt; b,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the handler to be called when the rate limit is reached.\n\nSynonyms:\nglimit.handler\nglimit handler","url":"glimit.html#handler"},{"doc":"glimit","title":"identifier","content":"pub fn identifier(\n  limiter: RateLimiterBuilder(a, b, c),\n  identifier: fn(a) -&gt; c,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the identifier function to be used to identify the rate limit.\n\nSynonyms:\nglimit.identifier\nglimit identifier","url":"glimit.html#identifier"},{"doc":"glimit","title":"new","content":"pub fn new() -&gt; RateLimiterBuilder(a, b, c)\n Create a new rate limiter builder.\n\nSynonyms:\nglimit.new\nglimit new","url":"glimit.html#new"},{"doc":"glimit","title":"per_hour","content":"pub fn per_hour(\n  limiter: RateLimiterBuilder(a, b, c),\n  limit: Int,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the rate limit per hour.\n\nSynonyms:\nglimit.per_hour\nglimit per_hour","url":"glimit.html#per_hour"},{"doc":"glimit","title":"per_minute","content":"pub fn per_minute(\n  limiter: RateLimiterBuilder(a, b, c),\n  limit: Int,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the rate limit per minute.\n\nSynonyms:\nglimit.per_minute\nglimit per_minute","url":"glimit.html#per_minute"},{"doc":"glimit","title":"per_second","content":"pub fn per_second(\n  limiter: RateLimiterBuilder(a, b, c),\n  limit: Int,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the rate limit per second.\n\nSynonyms:\nglimit.per_second\nglimit per_second","url":"glimit.html#per_second"},{"doc":"glimit","title":"try_build","content":"pub fn try_build(\n  config: RateLimiterBuilder(a, b, c),\n) -&gt; Result(RateLimiter(a, b, c), String)\n Build the rate limiter, but return an error instead of panicking.\n\nSynonyms:\nglimit.try_build\nglimit try_build","url":"glimit.html#try_build"},{"doc":"glimit","title":"glimit","content":" A framework-agnostic rate limiter for Gleam. 💫\n\n This module provides a rate limiter that can be used to limit the number of\n requests that can be made to a given function or handler within a given\n time frame.\n\n The rate limiter is implemented as an actor that keeps track of the number\n of hits for a given identifier within the last second, minute, and hour.\n When a hit is received, the actor checks the rate limits and either allows\n the hit to pass or rejects it.\n\n The rate limiter can be configured with rate limits per second, minute, and\n hour, and a handler function that is called when the rate limit is reached.\n The rate limiter can be applied to a function or handler using the `apply`\n function, which returns a new function that checks the rate limit before\n calling the original function.\n\n # Example\n\n ```gleam\n import glimit\n\n let limiter =\n   glimit.new()\n   |&gt; glimit.per_second(10)\n   |&gt; glimit.per_minute(100)\n   |&gt; glimit.per_hour(1000)\n   |&gt; glimit.identifier(fn(request) { request.ip })\n   |&gt; glimit.handler(fn(_request) { &quot;Rate limit reached&quot; })\n   |&gt; glimit.build()\n\n let handler =\n   fn(_request) { &quot;Hello, world!&quot; }\n   |&gt; glimit.apply(limiter)\n ```\n","url":"glimit.html"}]);